#Tested on Ubuntu
printf("==========================================================\n");
printf("A is real random sparse 20 x 20 \n"); 
printf("assert Q * R * U' - A  < 1e-10 \n");
printf("==========================================================\n");
n = 20;  d = 0.2;
A = sprandn (n,n,d) + speye (n,n);
[Q,R,E]= qr(A);
P=eye(size(E,1))(:,E);
assert (R'*R, (A*P)' * (A*P),1e-10)
assert (Q*R*P',A,1e-10)
printf("==========================================================\n");
printf("A is real random sparse 5 x 9 \n");
printf("assert Q * R * P' - A  < 1e-10 \n");
printf("==========================================================\n");
n = 9; m = 5; d = 0.3;
A = sprandn (m,n,d);
[Q,R,E]= qr(A);
P=eye(size(E,1))(:,E);
assert (Q*R*P',A,1e-10)
printf("==========================================================\n");
printf("A is complex random sparse 20 x 10 \n");
printf("assert Q * R * P' - A  < 1e-10 \n");
printf("==========================================================\n");
n = 10; m = 20; d = 0.3;
A = sprandn(m,n,d);
for j = 1:n
  for k = 1:m
    if A(k,j) != 0.0
      A(k,j) = A(k,j) + rand() * I; 
    endif
  endfor
endfor
[Q,R,E]= qr(A);
P=eye(size(E,1))(:,E);
assert (R'*R, (A*P)' * (A*P),1e-10)
assert (Q*R*P',A,1e-10)
printf("==========================================================\n");
printf("examples from bug 51950\n");
printf("==========================================================\n");
#example from comment#7 in https://savannah.gnu.org/bugs/?51950
A=sparse(5,6);
A(3,1)=0.8;
A(2,2)=1.4;
A(1,6)=-0.5;
[Q,R,E]=qr(A);
P=eye(size(E,1))(:,E);
tmp = qr(R);
Q*R*P' - A;
norm(Q*R*P' -A,2)

#example from original submission in https://savannah.gnu.org/bugs/?51950
#{
for iter = 1:100 
  A = sprandn(50,100,0.1);
  [Q,R,E] = qr(A);
  tmp = qr(R);
  P = eye(size(E,1))(:,E);
  norm(Q*R*P' -A, 1)
endfor
#}


printf("==========================================================\n");
printf("OVERDETERMINED SYSTEMS \n");
printf("==========================================================\n");

printf("==========================================================\n");
printf("least square solution for a real overdetermined system \n"); 
printf(" with dense rhs \n");
printf("==========================================================\n");
m = 20; n= 10; d = 0.3; 
A = sprandn (m,n,d);
b = ones(m,1);
x = qr(A,b);
norm(A * x - b,2)


printf("==========================================================\n");
printf("least square solution for a real overdetermined system \n"); 
printf(" with sparse rhs \n");
printf("==========================================================\n");
m = 20; n = 10; d = 0.2;
A = sprandn (m,n,d);
b = sprandn (m,3,0.4); 
x = qr(A,b);
norm(A * x - b,2)


printf("==========================================================\n");
printf("least square solution for a real overdetermined system \n"); 
printf(" with complex dense rhs \n");
printf("==========================================================\n");
m = 20; n = 10; d = 0.3; 
A = sprandn (m,n,d);
b  = randn(m,1); 
for k = 1:m
    if b(k,1) != 0.0
      b(k,1) = b(k,1) + rand() * I;
    endif 
endfor
x = qr(A,b);
norm(A * x - b,2)

printf("==========================================================\n");
printf("least square solution for a real overdetermined system \n"); 
printf("with complex sparse rhs \n");
printf("==========================================================\n");
m = 20; n=10; d = 0.3; 
A = sprandn(m,n,d);
b = sprandn(m,1,d); 
  for k = 1:m
      if b(k,1) != 0.0
        b(k,1) = b(k,1) + rand() * I;
      endif 
  endfor
x = qr(A,b);
norm(A * x - b,2)

printf("==========================================================\n");
printf("least square solution for a complex overdetermined system \n");
printf("with complex dense rhs \n");
printf("==========================================================\n");
m = 20; n = 10; d = 0.3;
A  = sprandn(m,n,0.3);
for j = 1:n
  for k = 1:m
    if A(k,j) != 0.0
      A(k,j) = A(k,j) + rand() * I; 
    endif
  endfor
endfor
b = randn(m,3);
for j = 1:3
  for k = 1:m
      b(k,j) = b(k,j) + rand() * I; 
  endfor
endfor
x = qr(A,b);
norm(A * x - b,2)


printf("==========================================================\n");
printf("least square solution for a complex overdetermined system \n");
printf("with real dense rhs \n");
printf("==========================================================\n");
m = 20; n = 10; d = 0.3;
A  = sprandn(m,n,0.3);
for j = 1:n
  for k = 1:m
    if A(k,j) != 0.0
      A(k,j) = A(k,j) + rand() * I; 
    endif
  endfor
endfor
b = randn(m,3);
x = qr(A,b);
norm(A * x - b,2)

printf("==========================================================\n");
printf("least square solution for a complex overdetermined system \n");
printf("with real sparse rhs \n");
printf("==========================================================\n");
m = 20; n = 10; d = 0.3;
A  = sprandn(m,n,0.3);
for j = 1:n
  for k = 1:m
    if A(k,j) != 0.0
      A(k,j) = A(k,j) + rand() * I; 
    endif
  endfor
endfor
b = sprandn (m,3,0.4);
x = qr(A,b);
norm(A * x - b,2)


printf("==========================================================\n");
printf("least square solution for a complex overdetermined system \n");
printf("with complex sparse rhs \n");
printf("==========================================================\n");
m = 20; n = 10; d = 0.3;
A = sprandn(m,n,d);
for j = 1:n
  for k = 1:m
    if A(k,j) != 0.0
      A(k,j) = A(k,j) + rand() * I; 
    endif
  endfor
endfor
b = sprandn(m,3,d); 
for j = 1:3
  for k = 1:m
      if b(k,j) != 0.0
        b(k,j) = b(k,j) + rand() * I;
      endif 
  endfor
endfor
x = qr(A,b);
norm(A * x - b,2)

#{
printf("==========================================================\n");
printf("least square solution for a real overdetermined system \n");
printf("with real dense rhs \n");
printf("test matrix is b1_ss.mtx from SuiteSparse\n");
printf("==========================================================\n");
#Copy directory spqr_test to your home directory. 
addpath("~/test_spqr/MatrixMarketUtils");
A=mmread("~/test_spqr/SPQRTestMatrices/b1_ss.mtx"); 
b = ones(7,1);
x = qr(A,b);
x;
norm(A * x - b,2)
#}

printf("==========================================================\n");
printf("UNDERDETERMINED SYSTEMS \n");
printf("==========================================================\n");


printf("==========================================================\n");
printf("min 2-norm solution for real underdetermined system\n");
printf("with real dense rhs \n");
printf("==========================================================\n");
m = 4; n = 10; d = 0.2;
A = sprandn (m,n,d);
b = ones(m,1); 
x = qr(A,b);
norm(A * x - b,2)


printf("==========================================================\n");
printf("min 2-norm solution for real underdetermined system\n");
printf("with real sparse rhs \n");
printf("==========================================================\n");
m = 4; n = 10; d = 0.2;
A = sprandn (m,n,d);
b = sprandn (m,3,d); 
x = qr(A,b);
norm(A * x - b,2)


printf("==========================================================\n");
printf("min 2-norm solution for real underdetermined system\n");
printf("with complex sparse rhs \n");
printf("==========================================================\n");
m = 4; n = 10; d = 0.2;
A = sprandn (m,n,d);
b = sprandn(m,3,d); 
for j = 1:3
  for k = 1:m
      if b(k,j) != 0.0
        b(k,j) = b(k,j) + rand() * I;
      endif 
  endfor
endfor
x = qr(A,b);
norm(A * x - b,2)

printf("==========================================================\n");
printf("min 2-norm solution for complex underdetermined system\n");
printf("with complex sparse rhs \n");
printf("==========================================================\n");
m = 4; n = 10; d = 0.2;
A = sprandn(m,n,d);
for j = 1:n
  for k = 1:m
    if A(k,j) != 0.0
      A(k,j) = A(k,j) + rand() * I; 
    endif
  endfor
endfor
b = sprandn(m,3,d);
for j = 1:3
  for k = 1:m
      if b(k,j) != 0.0
        b(k,j) = b(k,j) + rand() * I;
      endif 
  endfor
endfor
x = qr(A,b);
norm(A * x - b,2)

#{
printf("==========================================================\n");
printf("min 2-norm solution for underdetermined system\n");
printf("with real dense rhs \n");
printf("test matrix is lp_e226.mtx from SuiteSparse\n");
printf("==========================================================\n");
#Copy directory spqr_test to your home directory. 
addpath("~/test_spqr/MatrixMarketUtils");
A=mmread("~/test_spqr/SPQRTestMatrices/lp_e226.mtx"); 
A;
b = ones(size(A)(1),1);
x = qr(A,b);
norm(A * x - b,2)
#}

printf("==========================================================\n");
printf("DENSE QR \n");
printf("==========================================================\n");
A = rand(10,10);
b = rand(10,3);
[C,R] = qr(A,b); 
x = R\C;
norm(A * x - b,2) 
